/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
#include <config.h>
#include <bonobo/bonobo-i18n.h>
#include <gnome.h>
#include <libgnome/gnome-macros.h>
#include <string.h>
#include <libgnomevfs/gnome-vfs-uri.h>
#include <libgnomevfs/gnome-vfs-directory.h>
#include <libgnomevfs/gnome-vfs-ops.h>
#include <libgnomevfs/gnome-vfs-utils.h>

#include "gst-player-view.h"
#include "../libs/gst/play/play.h"
#include "../libs/gst/player/gstvideowidget.h"
#include "../libs/gst/player/gstplaylist.h"
#include "../libs/gst/player/gstcontrol.h"

static void     play_pause  (GtkWidget *widget, GstPlayerView *view);
static void     stop        (GtkWidget *widget, GstPlayerView *view);
static void     set_volume  (GtkWidget *widget, gdouble value, GstPlayerView *view);

struct GstPlayerViewDetails {
	GtkWidget  *vbox;
	GstPlay    *play;
	GtkWidget  *video_widget;
	GtkWidget  *control;
	GtkWidget  *playlist;
	char       *filename;
};

BONOBO_CLASS_BOILERPLATE (GstPlayerView, gst_player_view,
			  NautilusView, NAUTILUS_TYPE_VIEW)

static gboolean
is_audio_file (GnomeVFSFileInfo file_info)
{
	return eel_istr_has_prefix (file_info.mime_type, "audio/");
}

static void
start_uri (GstPlayerView *view, const char* location)
{
	view->details->filename = g_strdup_printf ("%s", location);
	gst_play_set_location (view->details->play, location);
}

static void
play_pause (GtkWidget *widget, GstPlayerView *view)
{
	GstElementState playstate;

	g_assert (GST_IS_PLAYER_VIEW (view));

	playstate = gst_play_get_state (view->details->play);

	if (playstate == GST_STATE_PAUSED)
	{
		gst_play_set_state (view->details->play, GST_STATE_PLAYING);
	}
	else if (playstate == GST_STATE_PLAYING)
	{
		gst_play_set_state (view->details->play, GST_STATE_PAUSED);
	}
	else
	{
		gst_play_set_state (view->details->play, GST_STATE_PLAYING);
	}
}

static void
stop (GtkWidget *widget, GstPlayerView *view)
{
	g_assert (GST_IS_PLAYER_VIEW (view));

	gst_play_set_state (view->details->play, GST_STATE_READY);
}

static void
set_volume (GtkWidget *widget, gdouble value, GstPlayerView *view)
{
	g_assert (GST_IS_PLAYER_VIEW (view));

	gst_play_set_volume (view->details->play, value);
}

static void
show_list (GtkWidget *widget, GstPlayerView *view)
{
	g_assert (GST_IS_PLAYER_VIEW (view));
	
	if (GTK_WIDGET_VISIBLE (view->details->playlist) == TRUE)
	{
		gtk_widget_hide (view->details->playlist);
	}
	else
	{
		gtk_widget_show (view->details->playlist);
	}
}

static void
set_selected (GtkWidget *widget, GstPlayerView *view)
{
	char *uri;

	uri = gst_playlist_get_selected (GST_PLAYLIST (view->details->playlist));
	start_uri (view, g_strdup_printf ("%s", uri));
}

static void
append_to_playlist (GstPlayerView *view, const char *location)
{
	gst_playlist_append (GST_PLAYLIST (view->details->playlist), location);

	if (view->details->filename == NULL)
	{
		gst_playlist_set_selected (GST_PLAYLIST (view->details->playlist), "0");
	}
}

static void
gst_player_view_finalize (GObject *object)
{
	GstPlayerView *view;
	
	view = GST_PLAYER_VIEW (object);
	
	g_free (view->details);

	G_OBJECT_CLASS (object)->finalize (object);
}

static void
load_location (GstPlayerView *view, const char *location)
{
	GnomeVFSResult    result;
	GnomeVFSFileInfo  file_info;
	char              *path_uri;
	char              *path;
	char              *message;

	g_assert (GST_IS_PLAYER_VIEW (view));

	result = gnome_vfs_get_file_info (location, &file_info,
					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE
					  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS);

	if (result != GNOME_VFS_OK) 
	{
		path = gnome_vfs_get_local_path_from_uri (location);
		message = g_strdup_printf (_("Sorry, but there was an error reading %s."), path);
		eel_show_error_dialog (message, _("Can't Read File"), 
                                       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view->details->vbox))));
		g_free (path);
		g_free (message);

		return;
	}

	if (is_audio_file (file_info))
		gtk_widget_hide (view->details->video_widget);
	
	path_uri = gnome_vfs_get_local_path_from_uri (location);

	append_to_playlist (view, g_strdup_printf ("%s", path_uri));
}

static void
sample_load_location_callback (NautilusView *nautilus_view, const char *location, gpointer user_data)
{
	GstPlayerView *view;
	
	g_assert (NAUTILUS_IS_VIEW (nautilus_view));
	g_assert (location != NULL);
	
	view = GST_PLAYER_VIEW (nautilus_view);
	
	nautilus_view_report_load_underway (nautilus_view);
	
	load_location (view, location);
	
	nautilus_view_report_load_complete (nautilus_view);
}

static void
bonobo_sample_callback (BonoboUIComponent *ui, 
			gpointer           user_data, 
			const char        *verb)
{
 	GstPlayerView *view;

	g_assert (BONOBO_IS_UI_COMPONENT (ui));
        g_assert (verb != NULL);

	view = GST_PLAYER_VIEW (user_data);

	if (strcmp (verb, "Sample Menu Item") == 0) {
	} else {
		g_assert (strcmp (verb, "Sample Dock Item") == 0);
	}
}

static void
sample_merge_bonobo_items_callback (BonoboControl *control, 
				    gboolean       state, 
				    gpointer       user_data)
{
 	GstPlayerView *view;
	BonoboUIComponent *ui_component;
	BonoboUIVerb verbs [] = {
		BONOBO_UI_VERB ("Sample Menu Item", bonobo_sample_callback),
		BONOBO_UI_VERB ("Sample Dock Item", bonobo_sample_callback),
		BONOBO_UI_VERB_END
	};

	g_assert (BONOBO_IS_CONTROL (control));
	
	view = GST_PLAYER_VIEW (user_data);

	if (state) {
		ui_component = nautilus_view_set_up_ui (NAUTILUS_VIEW (view),
							DATADIR,
							"gst-player-view-ui.xml",
							"gst-player-view");
									
		bonobo_ui_component_add_verb_list_with_data (ui_component, verbs, view);
	}
}

static void
gst_player_view_class_init (GstPlayerViewClass *class)
{
	G_OBJECT_CLASS (class)->finalize = gst_player_view_finalize;
}

static void
gst_player_view_instance_init (GstPlayerView *view)
{
	view->details = g_new0 (GstPlayerViewDetails, 1);
	
	view->details->play = gst_play_new (GST_PLAY_PIPE_VIDEO_THREADSAFE);
	
	view->details->vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (view->details->vbox);

	view->details->video_widget = gst_video_widget_new ();
	if (view->details->video_widget)
	{
		gtk_box_pack_start (GTK_BOX (view->details->vbox), view->details->video_widget, TRUE, TRUE, 0);
		gtk_widget_show (view->details->video_widget);
		/* FIXME update to new video widget */
		/*gst_play_set_video_sink (view->details->play, 
					 gst_video_widget_get_element (GST_VIDEO_WIDGET (view->details->video_widget)));*/
	} 
	else 
	{
		g_warning("could not find \"videosink\" element. Is it installed?");
	}

	view->details->playlist = gst_playlist_new ();
	gtk_box_pack_start (GTK_BOX (view->details->vbox), view->details->playlist, TRUE, TRUE, 0);
	gtk_widget_show (view->details->playlist);
	g_signal_connect (G_OBJECT (view->details->playlist), "selection_change",
			  G_CALLBACK (set_selected), view);

	view->details->control = gst_control_new ();
	gtk_box_pack_start (GTK_BOX (view->details->vbox), view->details->control, FALSE, TRUE, 0);
	gtk_widget_show (view->details->control);
	g_signal_connect (G_OBJECT (view->details->control), "button_play",
			  G_CALLBACK (play_pause), view);
	g_signal_connect (G_OBJECT (view->details->control), "button_stop",
			  G_CALLBACK (stop), view);
	g_signal_connect (G_OBJECT (view->details->control), "volume_change",
			  G_CALLBACK (set_volume), view);
	g_signal_connect (G_OBJECT (view->details->control), "button_list",
			  G_CALLBACK (show_list), view);
        
	nautilus_view_construct (NAUTILUS_VIEW (view), view->details->vbox);
	
	g_signal_connect (view, "load_location",
			  G_CALLBACK (sample_load_location_callback), NULL);

        g_signal_connect_object (nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view)), "activate",
				 G_CALLBACK (sample_merge_bonobo_items_callback), view, 0);

	view->details->filename = NULL;
}
